"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tyr_1 = require("../tyr");
/** @isomorphic */
class SecureError extends Error {
    constructor(value) {
        let message;
        if (typeof value === 'string') {
            message = value || 'Security violation';
            super(message);
        }
        else if (value && typeof value === 'object') {
            message = value.message || 'Security violation';
            super(message);
            for (const propName in value) {
                switch (propName) {
                    case 'message':
                        break;
                    default:
                        this[propName] = value[propName];
                        break;
                }
            }
        }
        else {
            super('Security violation');
        }
    }
    toPlain() {
        const { field, technical } = this;
        const o = { message: this.message };
        if (field)
            o.field = field.pathName;
        if (technical !== undefined)
            o.technical = technical;
        return o;
    }
    toString() {
        let s = '';
        if (this.technical)
            s += this.technical + '\n';
        s += this.message;
        const st = this.stack;
        if (st)
            s += '\n' + st;
        return s;
    }
}
exports.default = SecureError;
tyr_1.default.SecureError = SecureError;
//# sourceMappingURL=secureError.js.map